from .constance_views import *
from .default_views import *

__title__ = 'ska.contrib.django.ska.views'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'login',
    'constance_login',
)
