from __future__ import absolute_import

__title__ = 'ska.contrib.django.ska.apps'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('default_app_config',)

default_app_config = 'ska.contrib.django.ska.apps.Config'
