__title__ = 'ska.gettext'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2013-2018 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('_',)


def _(val):
    """For future integrations with gettext."""
    return val
